@@ -1,8 +1,8 @@
-/*  $Id: BurgerSpaceEngine.cpp,v 1.18 2004/05/02 04:55:47 sarrazip Exp $
+/*  $Id: BurgerSpaceEngine.cpp,v 1.32 2007/12/22 23:03:35 sarrazip Exp $
     BurgerSpaceEngine.cpp - Main engine
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2007 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #include "BurgerSpaceEngine.h"
@@ -27,6 +27,15 @@
 #include <flatzebra/PixmapLoadError.h>
 
 #include <assert.h>
+#include <fstream>
+#include <iomanip>
+
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <dirent.h>
+#include <time.h>
+#include <errno.h>
 
 
 /*  Tiles:
@@ -148,6 +157,7 @@
 #include "images/digit9.xpm"
 
 using namespace std;
+using namespace flatzebra;
 
 #define __(s) (s)  /* Gettext-like macro */
 
@@ -171,7 +181,8 @@ static const int
     ENEMY_SPEED_FACTOR = 3,
     SCORE_TTL = 20,
     NUM_INIT_LIVES = 3,
-    NUM_LEVELS = 6;
+    NUM_LEVELS = 6,
+    CORNER_TURN_TOLERANCE = 8;
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -245,11 +256,11 @@ getCoupleFromDirection(int direction, int amplitude)
 {
     switch (direction)
     {
-	case BurgerSpaceEngine::RIGHT:  return Couple(+amplitude, 0);
-	case BurgerSpaceEngine::UP   :  return Couple(0, -amplitude);
-	case BurgerSpaceEngine::LEFT :  return Couple(-amplitude, 0);
-	case BurgerSpaceEngine::DOWN :  return Couple(0, +amplitude);
-	default:                        assert(false); return Couple(0, 0);
+	case RIGHT:  return Couple(+amplitude, 0);
+	case UP   :  return Couple(0, -amplitude);
+	case LEFT :  return Couple(-amplitude, 0);
+	case DOWN :  return Couple(0, +amplitude);
+	default: assert(false); return Couple(0, 0);
     }
 }
 
@@ -261,8 +272,8 @@ getDirectionFromCouple(Couple v)
     assert((v.x != 0) ^ (v.y != 0));  // either x or y must be zero, not both
 
     if (v.x == 0)
-	return (v.y < 0 ? BurgerSpaceEngine::UP : BurgerSpaceEngine::DOWN);
-    return (v.x < 0 ? BurgerSpaceEngine::LEFT : BurgerSpaceEngine::RIGHT);
+	return (v.y < 0 ? UP : DOWN);
+    return (v.x < 0 ? LEFT : RIGHT);
 }
 
 
@@ -283,12 +294,12 @@ getDirectionName(int d)
 {
     switch (d)
     {
-	case BurgerSpaceEngine::RIGHT:  return "right";
-	case BurgerSpaceEngine::UP:     return "up";
-	case BurgerSpaceEngine::LEFT:   return "left";
-	case BurgerSpaceEngine::DOWN:   return "down";
-	case -1:  return "none";
-	default:  return "unknown";
+	case RIGHT: return "right";
+	case UP:    return "up";
+	case LEFT:  return "left";
+	case DOWN:  return "down";
+	case -1:    return "none";
+	default:    return "unknown";
     }
 }
 
@@ -856,7 +867,7 @@ BurgerSpaceEngine::Level::getXPMAtPixel(Couple pos) const
 
 BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
 					int initLevelNumber,
-					bool useSound,
+					bool _useSound,
 					bool fullScreen,
 					bool useOldMotionMode,
 					SDLKey pepperKey)
@@ -870,7 +881,7 @@ BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
     tickCount(0),
     tilePixmaps(),
 
-    initPlayerPos(0, 0),
+    initPlayerPos(),
     playerPA(18),
     playerSprite(NULL),
     lastPlayerDirection(-1),
@@ -902,34 +913,32 @@ BurgerSpaceEngine::BurgerSpaceEngine(const string &windowManagerCaption,
     digitPA(10),
     scoreSprites(),
 
-    quitKS(SDLK_ESCAPE),
-    leftKS(SDLK_LEFT),
-    rightKS(SDLK_RIGHT),
-    upKS(SDLK_UP),
-    downKS(SDLK_DOWN),
     startKS(SDLK_SPACE),
-    shootKS(pepperKey, SDLK_RCTRL),
+    quitKS(SDLK_ESCAPE),
+    theChefController(pepperKey),
     pauseKS(SDLK_p),
+    saveGameKS(SDLK_s),
+    loadGameKS(SDLK_l),
+    lastKeyPressed(SDLK_UNKNOWN),
 
     numHamburgersToDo(0),
     thePeakScore(0),
 
     theScore(0),
-    updateScore(true),
 
     celebrationMode(false),
 
     numLives(0),
-    updateNumLives(true),
     numAvailablePeppers(0),
-    updateNumAvailablePeppers(true),
 
     theCurrentLevel(),
 
-    theSoundMixer(NULL)
-{
-    this->useSound = useSound;
+    inSaveDialog(false),
+    inLoadDialog(false),
 
+    theSoundMixer(NULL),
+    useSound(_useSound)
+{
     assert(initLevelNumber >= 1);
     initLevelNo = initLevelNumber;
     cumulLevelNo = initLevelNo;
@@ -1066,6 +1075,7 @@ BurgerSpaceEngine::initializeMisc() throw(string)
     levelNoAreaPos =
 	Couple(getScreenWidthInPixels() - levelNoAreaSize.x, numLivesAreaPos.y);
 
+    initTimeForTreat();
 
     /*  Sound effects:
     */
@@ -1119,18 +1129,11 @@ void
 BurgerSpaceEngine::initGameParameters()
 {
     loadLevel(initLevelNo);
-    updateLevelNo = true;
 
     celebrationMode = false;
-
     theScore = 0;
-    updateScore = true;
-
     numAvailablePeppers = 5;
-    updateNumAvailablePeppers = true;
-
     numLives = 0;
-    updateNumLives = true;
 
     initTimeForTreat();
 }
@@ -1143,15 +1146,14 @@ BurgerSpaceEngine::initGameParameters()
 void
 BurgerSpaceEngine::processKey(SDLKey keysym, bool pressed)
 {
-    quitKS.check(keysym, pressed);
-    leftKS.check(keysym, pressed);
-    rightKS.check(keysym, pressed);
-    upKS.check(keysym, pressed);
-    downKS.check(keysym, pressed);
     startKS.check(keysym, pressed);
-    shootKS.check(keysym, pressed);
-    shootKS.check(keysym, pressed);
+    quitKS.check(keysym, pressed);
+    theChefController.check(keysym, pressed);
     pauseKS.check(keysym, pressed);
+    saveGameKS.check(keysym, pressed);
+    loadGameKS.check(keysym, pressed);
+    if (pressed)
+	lastKeyPressed = keysym;
 }
 
 
@@ -1162,11 +1164,28 @@ BurgerSpaceEngine::tick()
     if (quitKS.isPressed())
 	return false;
 
-    if (paused)
+    if (saveGameKS.justPressed())
+    {
+	drawSaveDialog();
+	inSaveDialog = true;
+    }
+
+    if (loadGameKS.justPressed())
+    {
+	drawLoadDialog();
+	inLoadDialog = true;
+    }
+
+    if (inSaveDialog)
+    {
+	doSaveDialog();
+    }
+    else if (inLoadDialog)
+    {
+	doLoadDialog();
+    }
+    else if (paused)
     {
-	restoreBackground();
-	drawSprites();
-	displayPauseMessage(true);
 	if (pauseKS.justPressed())
 	    paused = false;
     }
@@ -1177,15 +1196,21 @@ BurgerSpaceEngine::tick()
 	if (!animatePlayer())
 	    return false;
 
-	animateAutomaticCharacters();
-	restoreBackground();
-	drawSprites();
-	detectCollisions();
+	if (paused)
+	    displayPauseMessage(true);
+	else
+	{
+	    animateAutomaticCharacters();
+	    restoreBackground();
+	    drawSprites();
+	    detectCollisions();
+	}
     }
 
     startKS.remember();
-    shootKS.remember();
+    theChefController.remember();
     pauseKS.remember();
+    saveGameKS.remember();
 
     return true;
 }
@@ -1233,12 +1258,6 @@ BurgerSpaceEngine::initNextLevel(int levelNo /*= 0*/) throw(int)
 	    cumulLevelNo = levelNo;
 	loadLevel(cumulLevelNo);
 	initializeSprites();
-
-	updateLevelNo = true;
-	updateScore = true;
-	updateNumLives = true;
-	updateNumAvailablePeppers = true;
-
 	initTimeForTreat();
     }
     catch (string &msg)
@@ -1400,10 +1419,7 @@ BurgerSpaceEngine::getPlayerDesiredDirections(bool desiredDirs[4]) const
     allow us to have the chef be controlled by the computer.
 */
 {
-    desiredDirs[RIGHT] = rightKS.isPressed();
-    desiredDirs[UP]    = upKS.isPressed();
-    desiredDirs[LEFT]  = leftKS.isPressed();
-    desiredDirs[DOWN]  = downKS.isPressed();
+    theChefController.getDesiredDirections(desiredDirs);
 }
 
 
@@ -1466,7 +1482,6 @@ BurgerSpaceEngine::animatePlayer()
 		addToNumLives(-1);
 		assert(numLives >= 0);
 	    }
-
 	}
 
 	return true;
@@ -1503,7 +1518,7 @@ BurgerSpaceEngine::animatePlayer()
     /*  Shoot if requested:
     */
     static const bool infinitePepper = (getenv("INFINITEPEPPER") != NULL);
-    if (shootKS.justPressed())
+    if (theChefController.isShotRequested())
     {
 	if (infinitePepper || numAvailablePeppers > 0)
 	{
@@ -1533,8 +1548,6 @@ BurgerSpaceEngine::animatePlayer()
 				    Couple(0, 0), Couple(0, 0), size);
 	    s->setTimeToLive(12);
 	    pepperSprites.push_back(s);
-
-	    updateNumAvailablePeppers = true;
 	}
 	else
 	{
@@ -1685,7 +1698,7 @@ BurgerSpaceEngine::attemptMove(const Sprite &s,
 
     bool allowedDirections[4];
     Couple delta = determineAllowedDirections(
-			    s, speedFactor, speedFactor, allowedDirections);
+		    s, speedFactor, CORNER_TURN_TOLERANCE, allowedDirections);
 
     if (attemptLeft && allowedDirections[LEFT])
     {
@@ -1907,13 +1920,12 @@ BurgerSpaceEngine::animateAutomaticCharacters()
 	    delete *it;
 	    *it = NULL;
 
-	    playSoundEffect(treatDisappearsSound);
+	    if (numLives > 0)
+		playSoundEffect(treatDisappearsSound);
 	}
     removeNullElementsFromSpriteList(treatSprites);
 
-    bool newTreat = (--timeForTreat == 0);
-
-    if (newTreat)
+    if (timeForTreat == 0)
     {
 	initTimeForTreat();
 	Couple size = treatPA.getImageSize();
@@ -1937,8 +1949,11 @@ BurgerSpaceEngine::animateAutomaticCharacters()
 	s->setTimeToLive(5 * 20);
 	treatSprites.push_back(s);
 
-	playSoundEffect(treatAppearsSound);
+	if (numLives > 0)
+	    playSoundEffect(treatAppearsSound);
     }
+    else
+	timeForTreat--;
 }
 
 
@@ -2039,7 +2054,11 @@ BurgerSpaceEngine::carryEnemiesInList(
 	
 	if (ingredientGroupCollidesWithSprite(groupPos, groupSize, *enemy))
 	{
-	    g.addCarriedEnemy(enemy, INGREDIENT_FALL_SPEED);
+	    g.addCarriedEnemy(enemy);
+
+	    // Send the enemy downwards:
+	    enemy->setSpeed(Couple(0, INGREDIENT_FALL_SPEED));
+
 	    numCarriedEnemies++;
 	}
     }
@@ -2251,7 +2270,6 @@ void
 BurgerSpaceEngine::givePlayerPepper()
 {
     numAvailablePeppers++;
-    updateNumAvailablePeppers = true;
 }
 
 
@@ -2302,6 +2320,8 @@ BurgerSpaceEngine::detectCollisions()
 
     bool playIngredientLoweredSound = false;
 
+    assert(ingredientGroups.size() > 0);
+
     for (IngredientGroup::List::iterator it = ingredientGroups.begin();
 					it != ingredientGroups.end(); it++)
     {
@@ -2318,6 +2338,8 @@ BurgerSpaceEngine::detectCollisions()
 	*/
 	if (aGroup->getState() == IngredientGroup::NORMAL)
 	{
+	    assert(aGroup->getNumMembers() > 0);
+
 	    for (size_t j = 0; j < aGroup->getNumMembers(); j++)
 	    {
 		IngredientSprite *ing = aGroup->getMember(j);
@@ -2840,18 +2862,22 @@ BurgerSpaceEngine::drawSprites()
 	oscBit = 0;
 
 
+    /*  Ingredients:
+    */
+    for (IngredientSprite::List::const_iterator iti = ingredientSprites.begin();
+					iti != ingredientSprites.end(); iti++)
+    {
+	const IngredientSprite &is = **iti;
+	putSprite(is);
+    }
+
+
     /*  Treats:
     */
     SpriteList::const_iterator it;
     for (it = treatSprites.begin(); it != treatSprites.end(); it++)
 	putSprite(**it);
 
-    /*  Ingredients:
-    */
-    for (IngredientSprite::List::const_iterator iti = ingredientSprites.begin();
-					iti != ingredientSprites.end(); iti++)
-	putSprite(**iti);
-
 
     /*  "Complex" enemies (those with the normal number of images):
     */
@@ -2883,36 +2909,23 @@ BurgerSpaceEngine::drawSprites()
 
     /*  Score, number of lives and number of available peppers:
     */
-    updateScore = updateNumLives =
-		updateNumAvailablePeppers = updateLevelNo = true;
     char s[64];
-    if (updateScore)
-    {
-	snprintf(s, sizeof(s), "%s:%10ld", __("Score"), theScore);
-	writeString(s, scoreAreaPos);
-	updateScore = false;
-    }
-    if (updateNumLives)
-    {
-	snprintf(s, sizeof(s), "%s:%3d", __("Lives"), numLives);
-	writeString(s, numLivesAreaPos);
-	updateNumLives = false;
-    }
-    if (updateNumAvailablePeppers)
-    {
-	snprintf(s, sizeof(s), "%s:%3d", __("Peppers"), numAvailablePeppers);
-	writeString(s, numAvailablePeppersAreaPos);
-	updateNumAvailablePeppers = false;
-    }
-    if (updateLevelNo)
-    {
-	snprintf(s, sizeof(s), "%s:%3d", __("Level"), cumulLevelNo);
-	writeString(s, levelNoAreaPos);
-	updateLevelNo = false;
-    }
+    snprintf(s, sizeof(s), "%s:%10ld", __("Score"), theScore);
+    writeString(s, scoreAreaPos);
+    snprintf(s, sizeof(s), "%s:%3d", __("Lives"), numLives);
+    writeString(s, numLivesAreaPos);
+    snprintf(s, sizeof(s), "%s:%3d", __("Peppers"), numAvailablePeppers);
+    writeString(s, numAvailablePeppersAreaPos);
+    snprintf(s, sizeof(s), "%s:%3d", __("Level"), cumulLevelNo);
+    writeString(s, levelNoAreaPos);
 
     if (numLives == 0)
 	showInstructions();
+
+    const char *pressS = "[S] save game; [L] load game";
+    static const Couple fontdim = getFontDimensions();
+    int x = (theScreenSizeInPixels.x - strlen(pressS) * fontdim.x) / 2;
+    writeString(pressS, Couple(x, numLivesAreaPos.y));
 }
 
 
@@ -2971,7 +2984,6 @@ void
 BurgerSpaceEngine::addToScore(long n)
 {
     theScore += n;
-    updateScore = true;
 
     if (theScore > thePeakScore)
     {
@@ -2989,7 +3001,6 @@ void
 BurgerSpaceEngine::addToNumLives(int n)
 {
     numLives += n;
-    updateNumLives = true;
 }
 
 
@@ -3007,7 +3018,7 @@ BurgerSpaceEngine::displayPauseMessage(bool display)
 */
 {
     if (display)
-	displayMessage(2, "PAUSED -- press P to resume");
+	displayMessage(5, "PAUSED -- press P to resume");
 }
 
 
@@ -3058,3 +3069,711 @@ BurgerSpaceEngine::playSoundEffect(SoundMixer::Chunk &chunk)
 	}
     }
 }
+
+
+///////////////////////////////////////////////////////////////////////////////
+//
+// SAVING AND LOADING GAMES
+//
+
+
+int BurgerSpaceEngine::savedGameFormatVersion = 1;
+
+
+void
+BurgerSpaceEngine::drawSaveDialog()
+{
+    paused = true;
+    if (numLives > 0)
+	displayPauseMessage(true);
+    showDialogBox("SAVE GAME: which slot? (1-9, Q=cancel) ");
+    showSlotDirectory();
+}
+
+
+void
+BurgerSpaceEngine::drawLoadDialog()
+{
+    paused = true;
+    if (numLives > 0)
+	displayPauseMessage(true);
+    showDialogBox("LOAD GAME: which slot? (1-9, Q=cancel) ");
+    showSlotDirectory();
+}
+
+
+int
+BurgerSpaceEngine::getSlotNumber()
+{
+    if (lastKeyPressed >= SDLK_1 && lastKeyPressed <= SDLK_9)
+    {
+	int slotNum = int(lastKeyPressed - SDLK_1) + 1;
+	lastKeyPressed = SDLK_UNKNOWN;
+	return slotNum;
+    }
+
+    if (lastKeyPressed == SDLK_q)
+    {
+	lastKeyPressed = SDLK_UNKNOWN;
+	return -2;
+    }
+
+    return -1;
+}
+
+
+void
+BurgerSpaceEngine::doSaveDialog()
+{
+    int slotNum = getSlotNumber();
+
+    if (slotNum > 0)
+    {
+	char temp[512];
+	snprintf(temp, sizeof(temp), "Saving game in slot %d...", slotNum);
+	showDialogBox(temp);
+
+	string dir = getSavedGamesDir();
+	if (dir.empty())
+	{
+	    snprintf(temp, sizeof(temp),
+				"No usable directory for saved games.");
+	}
+	else
+	{
+	    string filename = dir + "/" + formSavedGameFilename(slotNum);
+	    ofstream out(filename.c_str());
+	    if (!out)
+	    {
+		int e = errno;
+		snprintf(temp, sizeof(temp), "Could not create slot %d.",
+								    slotNum);
+		fprintf(stderr, "%s: could not write to %s: %s\n",
+					PACKAGE, filename.c_str(), strerror(e));
+	    }
+	    else
+	    {
+		saveGame(out, slotNum);
+		snprintf(temp, sizeof(temp),
+			"Game saved in slot %d. Press P to resume.", slotNum);
+	    }
+	}
+	showDialogBox(temp);
+	inSaveDialog = false;
+	return;
+    }
+
+    if (slotNum == -2)
+    {
+	lastKeyPressed = SDLK_UNKNOWN;
+	showDialogBox("Game NOT saved.");
+	inSaveDialog = false;
+	return;
+    }
+}
+
+
+void
+BurgerSpaceEngine::doLoadDialog()
+{
+    int slotNum = getSlotNumber();
+
+    if (slotNum > 0)
+    {
+	char temp[512];
+	snprintf(temp, sizeof(temp), "Loading game from slot %d...", slotNum);
+	showDialogBox(temp);
+
+
+	string dir = getSavedGamesDir();
+	if (dir.empty())
+	{
+	    snprintf(temp, sizeof(temp),
+				"No usable directory. Press P to resume.");
+	}
+	else
+	{
+	    string filename = dir + "/" + formSavedGameFilename(slotNum);
+	    ifstream in(filename.c_str());
+	    if (!in)
+	    {
+		snprintf(temp, sizeof(temp),
+		    "No game saved in slot %d. Press P to resume.", slotNum);
+	    }
+	    else
+	    {
+		int errorCode = loadGame(in, slotNum);
+		if (errorCode != 0)
+		    snprintf(temp, sizeof(temp), "Load error: %d\n", errorCode);
+		else
+		{
+		    // Show the new game situation:
+		    restoreBackground();
+		    drawSprites();
+		    displayPauseMessage(true);
+
+		    snprintf(temp, sizeof(temp),
+			    "Game loaded from slot %d. Press P to resume.",
+								    slotNum);
+		}
+	    }
+	}
+	showDialogBox(temp);
+	inLoadDialog = false;
+	return;
+    }
+
+    if (slotNum == -2)
+    {
+	lastKeyPressed = SDLK_UNKNOWN;
+	showDialogBox("No game loaded. Press P to resume.");
+	inLoadDialog = false;
+	return;
+    }
+}
+
+
+void
+BurgerSpaceEngine::showDialogBox(const string &msg)
+{ 
+    static const Couple fontdim = getFontDimensions();
+    static const size_t maxlen = 50;
+    static const char *spaces = 
+		    "                                                      ";
+
+    string line = msg.substr(0, maxlen);
+    size_t margin = (maxlen + 4 - line.length()) / 2;
+    string blanks(spaces, margin);
+    line = blanks + line + blanks;
+
+    Couple pos((theScreenSizeInPixels.x - (maxlen + 4) * fontdim.x) / 2,
+		8 * fontdim.y);
+    writeString(spaces, pos + Couple(0, 0 * fontdim.y));
+    writeString(line,   pos + Couple(0, 1 * fontdim.y));
+    writeString(spaces, pos + Couple(0, 2 * fontdim.y));
+}
+
+
+string
+BurgerSpaceEngine::serialize(Couple c) const
+{
+    char temp[512];
+    snprintf(temp, sizeof(temp), "%d %d", c.x, c.y);
+    return temp;
+}
+
+
+int
+BurgerSpaceEngine::encodePixmapArrayPointer(const PixmapArray *ptr) const
+{
+    if (ptr == &playerPA)           return  0;
+    if (ptr == &pepperPA)           return  1;
+    if (ptr == &eggPA)              return  2;
+    if (ptr == &hotdogPA)           return  3;
+    if (ptr == &picklePA)           return  4;
+    if (ptr == &topBunPA)           return  5;
+    if (ptr == &lettucePA)          return  6;
+    if (ptr == &meatPA)             return  7;
+    if (ptr == &redStuffPA)         return  8;
+    if (ptr == &yellowStuffPA)      return  9;
+    if (ptr == &bottomBunPA)        return 10;
+    if (ptr == &treatPA)            return 11;
+    if (ptr == &digitPA)            return 12;
+    return -1;
+}
+
+const PixmapArray *
+BurgerSpaceEngine::decodePixmapArrayCode(int code) const
+{
+    if (code ==  0) return &playerPA;
+    if (code ==  1) return &pepperPA;
+    if (code ==  2) return &eggPA;
+    if (code ==  3) return &hotdogPA;
+    if (code ==  4) return &picklePA;
+    if (code ==  5) return &topBunPA;
+    if (code ==  6) return &lettucePA;
+    if (code ==  7) return &meatPA;
+    if (code ==  8) return &redStuffPA;
+    if (code ==  9) return &yellowStuffPA;
+    if (code == 10) return &bottomBunPA;
+    if (code == 11) return &treatPA;
+    if (code == 12) return &digitPA;
+    return NULL;
+}
+
+
+string
+BurgerSpaceEngine::serialize(const Sprite &s) const
+{
+    char temp[512];
+    Couple pos = s.getPos();
+    Couple speed = s.getSpeed();
+    Couple collBoxPos = s.getCollBoxPos();
+    Couple collBoxSize = s.getCollBoxSize();
+    snprintf(temp, sizeof(temp), "%d %d %d %d %d %d %d %d %d %d",
+			    pos.x, pos.y, 
+			    speed.x, speed.y,
+			    collBoxPos.x, collBoxPos.y,
+			    collBoxSize.x, collBoxSize.y,
+			    s.currentPixmapIndex,
+			    encodePixmapArrayPointer(s.getPixmapArray()));
+    return temp;
+}
+
+
+string
+BurgerSpaceEngine::serialize(const EnemySprite &es) const
+{
+    string s = serialize((Sprite &) es);
+    char temp[512];
+    snprintf(temp, sizeof(temp), " %d %d",
+			es.getDisappearanceTime(), es.getTimeToDie());
+    return s + temp;
+}
+
+
+void
+BurgerSpaceEngine::serializeSpriteList(ostream &out,
+					const SpriteList &list) const
+// The sprites are serialized in the order in which they appear in 'list'.
+{
+    out << list.size() << "\n";
+    for (SpriteList::const_iterator it = list.begin(); it != list.end(); it++)
+	out << serialize(**it) << "\n";
+}
+
+
+void
+BurgerSpaceEngine::serializeEnemySpriteList(ostream &out,
+					    const SpriteList &list) const
+// The sprites are serialized in the order in which they appear in 'list'.
+{
+    out << list.size() << "\n";
+    for (SpriteList::const_iterator it = list.begin(); it != list.end(); it++)
+	out << serialize(**it) << "\n";
+}
+
+
+string
+BurgerSpaceEngine::serialize(const IngredientSprite &is) const
+{
+    char temp[512];
+    snprintf(temp, sizeof(temp), "%d %d %d %d %d %d",
+		encodePixmapArrayPointer(is.getPixmapArray()),
+		is.getPos().x,
+		is.getPos().y,
+		is.getSpeed().x,
+		is.getSpeed().y,
+		is.isLowered());
+    return temp;
+}
+
+
+void
+BurgerSpaceEngine::saveGame(ostream &out, int slotNum)
+{
+    out << savedGameFormatVersion << "\n";
+    out << "\n";
+
+    out
+	<< initLevelNo << "\n"
+	<< cumulLevelNo << "\n"
+	<< tickCount << "\n"
+	<< serialize(initPlayerPos) << "\n"
+	<< serialize(*playerSprite) << "\n"
+	<< lastPlayerDirection << "\n"
+	<< oldMotionMode << "\n"
+	;
+
+    serializeSpriteList(out, pepperSprites);
+
+    out << timeForNewEnemy << "\n";
+
+    serializeEnemySpriteList(out, enemySprites);
+
+    out << "\n";
+    out << ingredientGroups.size() << "\n";
+    for (IngredientGroup::List::const_iterator it = ingredientGroups.begin();
+					    it != ingredientGroups.end(); it++)
+    {
+	const IngredientGroup &group = **it;
+	out << "\n";
+
+	out << group.getVerticalTarget() << " "
+	    << group.getBounceTime() << " "
+	    << group.getStallTime() << " "
+	    << (int) group.getState() << " "
+	    << group.getNumFloorsToGo() << " "
+	    << group.isTopBun() << "\n";
+
+	// Write indexes in enemySprites for each carried enemy:
+	const SpriteList &enemies = group.getCarriedEnemies();
+	out << enemies.size();
+	for (SpriteList::const_iterator jt = enemies.begin();
+						jt != enemies.end(); jt++)
+	{
+	    SpriteList::iterator kt = find(
+				enemySprites.begin(), enemySprites.end(), *jt);
+	    assert(kt != enemySprites.end());
+	    size_t index = size_t(distance(enemySprites.begin(), kt));
+	    out << " " << index;
+	}
+	out << "\n";
+
+	for (size_t i = 0; i < 4; i++)
+	    out << serialize(*group.getMember(i)) << "\n";
+    }
+    out << "\n";
+
+    serializeSpriteList(out, treatSprites);
+    out << timeForTreat << "\n";
+
+    serializeSpriteList(out, scoreSprites);
+
+    out
+	<< numHamburgersToDo << "\n"
+	<< thePeakScore << "\n"
+	<< theScore << "\n"
+	<< celebrationMode << "\n"
+	<< numLives << "\n"
+	<< numAvailablePeppers << "\n"
+	;
+}
+
+
+bool
+BurgerSpaceEngine::deserialize(istream &in, Couple &c) const
+{
+    in >> c.x >> c.y;
+    return in.good();
+}
+
+
+Sprite *
+BurgerSpaceEngine::deserializeSprite(istream &in, bool enemy) const
+{
+    Couple pos, speed, collBoxPos, collBoxSize;
+    int pixmapIndex, pixmapArrayCode;
+    in >> pos.x >> pos.y
+	>> speed.x >> speed.y
+	>> collBoxPos.x >> collBoxPos.y
+	>> collBoxSize.x >> collBoxSize.y
+	>> pixmapIndex
+	>> pixmapArrayCode;
+    if (!in)
+	return NULL;
+
+    const PixmapArray *pa = decodePixmapArrayCode(pixmapArrayCode);
+    if (pa == NULL)
+	return NULL;
+
+    Sprite *s;
+    if (enemy)
+	s = new EnemySprite(*pa, pos, speed, collBoxPos, collBoxSize);
+    else
+	s = new Sprite(*pa, pos, speed, Couple(), collBoxPos, collBoxSize);
+
+    s->currentPixmapIndex = pixmapIndex;
+    return s;
+}
+
+
+IngredientSprite *
+BurgerSpaceEngine::deserializeIngredientSprite(istream &in,
+						IngredientGroup *ig) const
+{
+    int pixmapArrayCode, x, y, dx, dy, isLowered;
+    in >> pixmapArrayCode >> x >> y >> dx >> dy >> isLowered;
+    if (!in)
+	return NULL;
+
+    const PixmapArray *pa = decodePixmapArrayCode(pixmapArrayCode);
+    if (pa == NULL)
+	return NULL;
+
+    IngredientSprite *is = new IngredientSprite(*pa, Couple(x, y), ig);
+    is->setSpeed(Couple(dx, dy));
+    if (isLowered)
+	is->setLowered();
+    return is;
+}
+
+
+bool
+BurgerSpaceEngine::deserializeSpriteList(istream &in,
+					SpriteList &list,
+					bool enemies) const
+{
+    size_t numSprites;
+    in >> numSprites;
+    if (!in || numSprites < 0 || numSprites > 9999)
+	return false;
+
+    list.clear();
+    for (size_t i = 0; i < numSprites; i++)
+    {
+	Sprite *s = deserializeSprite(in, enemies);
+	if (s == NULL)
+	{
+	    deleteSprites(list);
+	    return false;
+	}
+	list.push_back(s);
+    }
+    return true;
+}
+
+
+int
+BurgerSpaceEngine::loadGame(istream &in, int slotNum)
+{
+    int version;
+    in >> version;
+    if (!in)
+	return -1;
+    if (version != savedGameFormatVersion)
+	return -2;
+
+    in >> initLevelNo >> cumulLevelNo >> tickCount;
+    if (!in)
+	return -3;
+
+    initNextLevel(cumulLevelNo);
+
+    if (!deserialize(in, initPlayerPos))
+	return -4;
+
+    delete playerSprite;
+    playerSprite = deserializeSprite(in);
+    if (playerSprite == NULL)
+	return -5;
+
+    in >> lastPlayerDirection >> oldMotionMode;
+    if (!in)
+	return -6;
+
+    if (!deserializeSpriteList(in, pepperSprites))
+	return -7;
+
+    in >> timeForNewEnemy;
+    if (!in)
+	return -8;
+
+    if (!deserializeSpriteList(in, enemySprites, true))
+	return -9;
+
+    size_t numIngredientGroups;
+    in >> numIngredientGroups;
+    if (!in)
+	return -10;
+
+    ingredientSprites.clear();
+    ingredientGroups.clear();
+
+    for (size_t i = 0; i < numIngredientGroups; i++)
+    {
+	int verticalTarget, bounceTime, stallTime, state;
+	size_t numFloorsToGo;
+	bool isTopBun;
+	in >> verticalTarget >> bounceTime >> stallTime
+			>> state >> numFloorsToGo >> isTopBun;
+	if (!in)
+	    return -11;
+
+	size_t numCarriedEnemies;
+	in >> numCarriedEnemies;
+	if (!in)
+	    return -12;
+
+	vector<size_t> enemyIndexes;
+	for (size_t j = 0; j < numCarriedEnemies; j++)
+	{
+	    size_t index;
+	    in >> index;
+	    if (!in || index < 0 || index >= enemySprites.size())
+		return -13;
+	    enemyIndexes.push_back(index);
+	}
+
+	IngredientGroup *group = new IngredientGroup(verticalTarget, isTopBun);
+	group->restore(bounceTime, stallTime,
+			IngredientGroup::State(state), numFloorsToGo);
+
+	// Read and insert the IngredientSprite members of this group:
+	for (size_t k = 0; k < 4; k++)
+	{
+	    IngredientSprite *is = deserializeIngredientSprite(in, group);
+	    if (is == NULL)
+	    {
+		delete group;
+		return -14;
+	    }
+	    group->setMember(k, is);
+	    is->currentPixmapIndex = k;
+	    ingredientSprites.push_back(is);
+	}
+
+	// Tell each carried enemy by which group it is carried:
+	for (vector<size_t>::const_iterator jt = enemyIndexes.begin();
+					    jt != enemyIndexes.end(); jt++)
+	{
+	    size_t enemyIndex = *jt;
+	    Sprite *s = enemySprites.at(enemyIndex);
+	    EnemySprite *es = dynamic_cast<EnemySprite *>(s);
+	    if (s == NULL || es == NULL)
+	    {
+		delete group;
+		return -15;
+	    }
+	    group->addCarriedEnemy(es);
+	}
+
+
+	ingredientGroups.push_back(group);
+    }
+
+    if (!deserializeSpriteList(in, treatSprites))
+	return -16;
+    in >> timeForTreat;
+    if (!in)
+	return -17;
+
+    if (!deserializeSpriteList(in, scoreSprites))
+	return -18;
+
+    in
+	>> numHamburgersToDo
+	>> thePeakScore
+	>> theScore
+	>> celebrationMode
+	>> numLives
+	>> numAvailablePeppers
+	;
+    if (!in)
+	return -19;
+
+    return 0;
+}
+
+
+/*static*/
+string
+BurgerSpaceEngine::getSavedGamesDir()
+/*  Returns the full path of a directory in which files can be created
+    for saved games.  The path does not finish with a '/'.
+    If no such path could be formed, an empty string is returned.
+*/
+{
+    const char *home = getenv("HOME");  // TODO: what about non-Unix systems?
+    if (home == NULL)
+	return "";
+    
+    struct stat statbuf;
+    if (stat(home, &statbuf) != 0)
+    {
+	int e = errno;
+	fprintf(stderr, "%s: stat(%s): %s\n", PACKAGE, home, strerror(e));
+	return "";
+    }
+    if (!S_ISDIR(statbuf.st_mode))
+    {
+	fprintf(stderr, "%s: $HOME (%s) is not a directory\n", PACKAGE, home);
+	return "";
+    }
+
+    string subdir = home + string("/") + "." + PACKAGE;
+    if (stat(subdir.c_str(), &statbuf) != 0)
+    {
+	if (mkdir(subdir.c_str(), 0700) != 0)
+	{
+	    int e = errno;
+	    fprintf(stderr, "%s: mkdir(%s): %s\n",
+					PACKAGE, subdir.c_str(), strerror(e));
+	    return "";
+	}
+    }
+    else if (!S_ISDIR(statbuf.st_mode))
+    {
+	fprintf(stderr, "%s: %s is not a directory\n", PACKAGE, subdir.c_str());
+	return "";
+    }
+
+    if (access(subdir.c_str(), W_OK) != 0)
+    {
+	fprintf(stderr, "%s: %s is not a writable directory\n",
+						PACKAGE, subdir.c_str());
+	return "";
+    }
+
+    return subdir;
+}
+
+
+/*static*/
+string
+BurgerSpaceEngine::formSavedGameFilename(int slotNum)
+{
+    char temp[512];
+    snprintf(temp, sizeof(temp), "saved-game-%d.dat", slotNum);
+    return temp;
+}
+
+
+void
+BurgerSpaceEngine::showSlotDirectory()
+{
+    vector<string> slotTable;
+    slotTable.push_back("");
+    slotTable.push_back("No  Save Time");
+    slotTable.push_back("--  -------------------");
+
+    string dirname = getSavedGamesDir();
+
+    // Scan the saved games directory and build a table of available games:
+    for (size_t slotNum = 1; slotNum <= 9; slotNum++)
+    {
+	string slotDesc = string(" ") + char('0' + slotNum) + "  ";
+	string filename = dirname + "/" + formSavedGameFilename(slotNum);
+	ifstream file(filename.c_str());
+	if (!file)
+	    slotDesc += "(empty)";
+	else
+	{
+	    struct stat statbuf;
+	    if (stat(filename.c_str(), &statbuf) != 0)
+		slotDesc += "(empty)";
+	    else
+	    {
+		struct tm t = *localtime(&statbuf.st_mtime);
+		char fileTime[512];
+		strftime(fileTime, sizeof(fileTime), "%Y-%m-%d %H:%M:%S", &t);
+		slotDesc += fileTime;
+	    }
+	}
+	slotTable.push_back(slotDesc);
+    }
+
+    slotTable.push_back("--  -------------------");
+    slotTable.push_back("");
+
+    // Find length of longest line:
+    size_t maxlen = 0;
+    for (vector<string>::const_iterator it = slotTable.begin();
+					    it != slotTable.end(); it++)
+	maxlen = max(maxlen, it->length());
+
+    // Write the table:
+    static const Couple fontdim = getFontDimensions();
+    static const char *spaces =
+       "                                                                      ";
+    Couple pos((theScreenSizeInPixels.x - (maxlen + 4) * fontdim.x) / 2,
+		12 * fontdim.y);
+
+    size_t row = 0;
+    for (vector<string>::const_iterator it = slotTable.begin();
+					    it != slotTable.end(); it++, row++)
+    {
+	string s = "  " + *it + string(spaces, maxlen + 2 - it->length());
+	writeString(s, pos + Couple(0, row * fontdim.y));
+    }
+}
