@@ -1,8 +1,8 @@
-/*  $Id: BurgerSpaceEngine.h,v 1.16 2004/05/02 04:55:47 sarrazip Exp $
+/*  $Id: BurgerSpaceEngine.h,v 1.26 2007/12/22 23:03:35 sarrazip Exp $
     BurgerSpaceEngine.h - Main engine
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2007 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #ifndef _H_BurgerSpaceEngine
@@ -25,6 +25,7 @@
 
 #include "IngredientGroup.h"
 #include "IngredientSprite.h"
+#include "ChefController.h"
 
 #include <flatzebra/GameEngine.h>
 #include <flatzebra/Sprite.h>
@@ -32,9 +33,10 @@
 #include <flatzebra/KeyState.h>
 
 #include <string>
+#include <iostream>
 
 
-class BurgerSpaceEngine : public GameEngine
+class BurgerSpaceEngine : public flatzebra::GameEngine
 /*  A hamburger-smashing video game.
 */
 {
@@ -42,12 +44,6 @@ public:
 
     typedef char **XPM;
 
-    enum { RIGHT, UP, LEFT, DOWN };
-    /*	Possible directions of movement, in trigonometric order.
-	This enumeration is garanteed to start at zero.
-	The order of the constants is guaranteed, so they are 0, 1, 2, 3.
-    */
-
     BurgerSpaceEngine(const std::string &windowManagerCaption,
 			int initLevelNumber,
 			bool useSound,
@@ -109,20 +105,20 @@ private:
     class Level
     {
     public:
-	Couple sizeInTiles;
-	Couple sizeInPixels;
-	Couple positionInPixels;
+	flatzebra::Couple sizeInTiles;
+	flatzebra::Couple sizeInPixels;
+	flatzebra::Couple positionInPixels;
 
 	Level();
 	~Level();
 
-	void init(int no, int nCols, int nRows, Couple posInPixels);
+	void init(int no, int nCols, int nRows, flatzebra::Couple posInPixels);
 	void setLevelNo(int no);
 	int  getLevelNo() const;
 	void setTileMatrixEntry(int colNo, int rowNo,
 				XPM xpm, SDL_Surface *pixmap);
 	SDL_Surface **getTileMatrixRow(int rowNo);
-	XPM getXPMAtPixel(Couple pos) const;
+	XPM getXPMAtPixel(flatzebra::Couple pos) const;
 
     private:
 	int levelNo;
@@ -190,15 +186,14 @@ private:
 
     /*  PLAYER:
     */
-    bool userControlledPlayer;
-    Couple initPlayerPos;  // initial player position in a level
-    PixmapArray playerPA;
-    Sprite *playerSprite;
+    flatzebra::Couple initPlayerPos;  // initial player position in a level
+    flatzebra::PixmapArray playerPA;
+    flatzebra::Sprite *playerSprite;
     int lastPlayerDirection;
     bool oldMotionMode;
 
-    PixmapArray pepperPA;
-    SpriteList  pepperSprites;
+    flatzebra::PixmapArray pepperPA;
+    flatzebra::SpriteList  pepperSprites;
 
 
     /*  ENEMIES:
@@ -207,93 +202,94 @@ private:
     		// tick count at which a new enemy must be created;
 		// 0 means none
 
-    PixmapArray eggPA;
-    PixmapArray hotdogPA;
-    PixmapArray picklePA;
-    SpriteList  enemySprites;
+    flatzebra::PixmapArray eggPA;
+    flatzebra::PixmapArray hotdogPA;
+    flatzebra::PixmapArray picklePA;
+    flatzebra::SpriteList  enemySprites;
 
 
     /*  INGREDIENTS:
     */
-    PixmapArray topBunPA;
-    PixmapArray lettucePA;
-    PixmapArray meatPA;
-    PixmapArray redStuffPA;
-    PixmapArray yellowStuffPA;
-    PixmapArray bottomBunPA;
+    flatzebra::PixmapArray topBunPA;
+    flatzebra::PixmapArray lettucePA;
+    flatzebra::PixmapArray meatPA;
+    flatzebra::PixmapArray redStuffPA;
+    flatzebra::PixmapArray yellowStuffPA;
+    flatzebra::PixmapArray bottomBunPA;
     IngredientSprite::List ingredientSprites;  // owns the contained objects
     IngredientGroup::List  ingredientGroups;
+				    // contained objects must come from 'new'
 
 
     /*  TREATS (icecream, etc, that the player eats to get a pepper):
     */
-    PixmapArray treatPA;
-    SpriteList treatSprites;
+    flatzebra::PixmapArray treatPA;
+    flatzebra::SpriteList treatSprites;
     int timeForTreat;
 
 
     /*  DIGITS:
     */
-    PixmapArray digitPA;
-    SpriteList scoreSprites;
+    flatzebra::PixmapArray digitPA;
+    flatzebra::SpriteList scoreSprites;
 
 
     /*  KEYBOARD COMMANDS:
     */
-    KeyState quitKS;
-    KeyState leftKS;
-    KeyState rightKS;
-    KeyState upKS;
-    KeyState downKS;
-    KeyState startKS;
-    KeyState shootKS;
-    KeyState pauseKS;
+    flatzebra::KeyState startKS;
+    flatzebra::KeyState quitKS;
+    ChefController theChefController;
+    flatzebra::KeyState pauseKS;
+    flatzebra::KeyState saveGameKS;
+    flatzebra::KeyState loadGameKS;
+    SDLKey lastKeyPressed;
 
 
     int    numHamburgersToDo;
     long   thePeakScore;  // player's best score yet in this game
 
     long   theScore;      // player's score in points
-    bool   updateScore;   // indicates if player's score needs to be rewritten
-    Couple scoreAreaPos;
-    Couple scoreAreaSize;
+    flatzebra::Couple scoreAreaPos;
+    flatzebra::Couple scoreAreaSize;
 
     bool   celebrationMode;  // used when player has just won the level
 
     int    numLives;  // number of player lives left
-    bool   updateNumLives;
-    Couple numLivesAreaPos;
-    Couple numLivesAreaSize;
+    flatzebra::Couple numLivesAreaPos;
+    flatzebra::Couple numLivesAreaSize;
 
     int    numAvailablePeppers;  // number of pepper shots available to player
-    bool   updateNumAvailablePeppers;
-    Couple numAvailablePeppersAreaPos;
-    Couple numAvailablePeppersAreaSize;
+    flatzebra::Couple numAvailablePeppersAreaPos;
+    flatzebra::Couple numAvailablePeppersAreaSize;
 
-    bool   updateLevelNo;
-    Couple levelNoAreaPos;
-    Couple levelNoAreaSize;
+    flatzebra::Couple levelNoAreaPos;
+    flatzebra::Couple levelNoAreaSize;
 
     Level theCurrentLevel;
 
+    bool inSaveDialog;
+    bool inLoadDialog;
+
+    static int savedGameFormatVersion;
+
     /*  SOUND EFFECTS:
     */
-    SoundMixer *theSoundMixer;  // see method playSoundEffect()
+    flatzebra::SoundMixer *theSoundMixer;  // see method playSoundEffect()
     bool useSound;
-    SoundMixer::Chunk ingredientBouncesSound;
-    SoundMixer::Chunk ingredientInPlateSound;
-    SoundMixer::Chunk ingredientFallsSound;
-    SoundMixer::Chunk ingredientLoweredSound;
-    SoundMixer::Chunk enemyCatchesChefSound;
-    SoundMixer::Chunk enemyParalyzedSound;
-    SoundMixer::Chunk enemySmashedSound;
-    SoundMixer::Chunk chefThrowsPepperSound;
-    SoundMixer::Chunk chefGetsTreatSound;
-    SoundMixer::Chunk chefShootsBlanksSound;
-    SoundMixer::Chunk newGameStartsSound;
-    SoundMixer::Chunk levelFinishedSound;
-    SoundMixer::Chunk treatAppearsSound;
-    SoundMixer::Chunk treatDisappearsSound;
+    flatzebra::SoundMixer::Chunk ingredientBouncesSound;
+    flatzebra::SoundMixer::Chunk ingredientInPlateSound;
+    flatzebra::SoundMixer::Chunk ingredientFallsSound;
+    flatzebra::SoundMixer::Chunk ingredientLoweredSound;
+    flatzebra::SoundMixer::Chunk enemyCatchesChefSound;
+    flatzebra::SoundMixer::Chunk enemyParalyzedSound;
+    flatzebra::SoundMixer::Chunk enemySmashedSound;
+    flatzebra::SoundMixer::Chunk chefThrowsPepperSound;
+    flatzebra::SoundMixer::Chunk chefGetsTreatSound;
+    flatzebra::SoundMixer::Chunk chefShootsBlanksSound;
+    flatzebra::SoundMixer::Chunk newGameStartsSound;
+    flatzebra::SoundMixer::Chunk levelFinishedSound;
+    flatzebra::SoundMixer::Chunk treatAppearsSound;
+    flatzebra::SoundMixer::Chunk treatDisappearsSound;
 
 
     ///////////////////////////////////////////////////////////////////////////
@@ -303,7 +299,7 @@ private:
     //
     void chooseDirectionAmongMany(bool directions[4]) const;
     int  chooseDirectionTowardTarget(
-			    Couple startPos, Couple targetPos, int speedFactor,
+			    flatzebra::Couple startPos, flatzebra::Couple targetPos, int speedFactor,
 			    const bool allowedDirections[4]) const;
     void avoidEnemies(bool desiredDirs[4]) const;
     void getPlayerDesiredDirections(bool desiredDirs[4]) const;
@@ -314,57 +310,88 @@ private:
     void restoreBackground();
     void drawSprites();
 
-    void putSprite(const Sprite &s);
+    void putSprite(const flatzebra::Sprite &s);
     void showInstructions();
     void initGameParameters();
     void initNextLevel(int levelNo = 0) throw(int);
     void startNewLife();
-    void animateTemporarySprites(SpriteList &slist) const;
+    void animateTemporarySprites(flatzebra::SpriteList &slist) const;
     void givePlayerPepper();
     void makePlayerWin();
     void makePlayerDie();
     void releaseAllCarriedEnemies();
-    void detectEnemyCollisions(SpriteList &slist);
+    void detectEnemyCollisions(flatzebra::SpriteList &slist);
     void detectCollisionBetweenIngredientGroupAndEnemyList(
-	    const IngredientGroup &aGroup, SpriteList &enemies);
+	    const IngredientGroup &aGroup, flatzebra::SpriteList &enemies);
     bool ingredientGroupCollidesWithSprite(
-			const Couple groupPos, const Couple groupSize,
-			const Sprite &s) const;
-    void loadPixmaps() throw(PixmapLoadError);
+			const flatzebra::Couple groupPos, const flatzebra::Couple groupSize,
+			const flatzebra::Sprite &s) const;
+    void loadPixmaps() throw(flatzebra::PixmapLoadError);
     void loadLevel(int levelNo) throw(std::string);
     void displayErrorMessage(const std::string &msg) throw();
-    void initializeSprites() throw(PixmapLoadError);
+    void initializeSprites() throw(flatzebra::PixmapLoadError);
     void initializeMisc() throw(std::string);
-    void killSpritesInList(SpriteList &sl);
+    void killSpritesInList(flatzebra::SpriteList &sl);
     IngredientGroup *findIngredientGroupRightBelow(
 					const IngredientGroup &upperGroup);
     bool isIngredientSpriteOnFloor(const IngredientSprite &s) const;
     bool spriteTouchesIngredientGroup(
-			    const Sprite &s, const IngredientGroup &g) const;
+			    const flatzebra::Sprite &s, const IngredientGroup &g) const;
     size_t carryEnemies(IngredientGroup &g);
-    size_t carryEnemiesInList(IngredientGroup &g, SpriteList &slist);
+    size_t carryEnemiesInList(IngredientGroup &g, flatzebra::SpriteList &slist);
     size_t releaseCarriedEnemies(IngredientGroup &g);
-    void createScoreSprites(long n, Couple center);
-    void moveEnemyList(SpriteList &slist, int speedFactor);
-    Couple getDistanceToPerfectPos(const Sprite &s) const;
-    Couple determineAllowedDirections(const Sprite &s,
+    void createScoreSprites(long n, flatzebra::Couple center);
+    void moveEnemyList(flatzebra::SpriteList &slist, int speedFactor);
+    flatzebra::Couple getDistanceToPerfectPos(const flatzebra::Sprite &s) const;
+    flatzebra::Couple determineAllowedDirections(const flatzebra::Sprite &s,
 				    int speedFactor, int tolerance,
 				    bool allowedDirections[4]) const;
-    Couple attemptMove(const Sprite &s, bool attemptLeft, bool attemptRight,
+    flatzebra::Couple attemptMove(const flatzebra::Sprite &s, bool attemptLeft, bool attemptRight,
 			bool attemptUp, bool attemptDown,
 			int speedFactor) const;
-    bool positionAllowsLeftMovement(Couple pos, Couple size) const;
-    bool positionAllowsRightMovement(Couple pos, Couple size) const;
-    bool spriteBottomCenterIsOnLadder(const Sprite &s) const;
-    bool spriteBottomCenterIsOverLadder(const Sprite &s) const;
-    void drawComplexEnemySprites(const SpriteList &slist, int oscBit);
+    bool positionAllowsLeftMovement(flatzebra::Couple pos, flatzebra::Couple size) const;
+    bool positionAllowsRightMovement(flatzebra::Couple pos, flatzebra::Couple size) const;
+    bool spriteBottomCenterIsOnLadder(const flatzebra::Sprite &s) const;
+    bool spriteBottomCenterIsOverLadder(const flatzebra::Sprite &s) const;
+    void drawComplexEnemySprites(const flatzebra::SpriteList &slist, int oscBit);
     void addToScore(long n);
     void addToNumLives(int n);
     void initTimeForTreat();
     void displayPauseMessage(bool display);
     void displayStartMessage(bool display);
     void displayMessage(int row, const char *msg);
-    void playSoundEffect(SoundMixer::Chunk &wb);
+    void playSoundEffect(flatzebra::SoundMixer::Chunk &wb);
+
+    void drawSaveDialog();
+    void drawLoadDialog();
+    int  getSlotNumber();
+    void doSaveDialog();
+    void doLoadDialog();
+    void showDialogBox(const std::string &msg);
+
+    std::string serialize(flatzebra::Couple c) const;
+    std::string serialize(const flatzebra::Sprite &s) const;
+    std::string serialize(const EnemySprite &s) const;
+    int encodePixmapArrayPointer(const flatzebra::PixmapArray *ptr) const;
+    const flatzebra::PixmapArray *decodePixmapArrayCode(int) const;
+    void serializeSpriteList(std::ostream &out,
+				    const flatzebra::SpriteList &list) const;
+    void serializeEnemySpriteList(std::ostream &out,
+				    const flatzebra::SpriteList &list) const;
+    std::string serialize(const IngredientSprite &is) const;
+    void saveGame(std::ostream &out, int slotNum);
+    bool deserialize(std::istream &in, flatzebra::Couple &c) const;
+    flatzebra::Sprite *deserializeSprite(std::istream &in,
+					bool enemy = false) const;
+    bool deserializeSpriteList(std::istream &in,
+				flatzebra::SpriteList &list,
+				bool enemies = false) const;
+    IngredientSprite *deserializeIngredientSprite(std::istream &in,
+						IngredientGroup *ig) const;
+    int loadGame(std::istream &in, int slotNum);
+    static std::string getSavedGamesDir();
+    static std::string formSavedGameFilename(int slotNum);
+    void showSlotDirectory();
 
 
     /*	Forbidden operations:
