@@ -1,8 +1,8 @@
-/*  $Id: EnemySprite.cpp,v 1.4 2004/05/02 04:55:47 sarrazip Exp $
+/*  $Id: EnemySprite.cpp,v 1.6 2007/12/22 23:03:35 sarrazip Exp $
     EnemySprite.cpp - Sprite that represents an enemy of the player.
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2007 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,14 +16,16 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #include "EnemySprite.h"
 
 #include <stdio.h>
 
+using namespace flatzebra;
+
 
 EnemySprite::EnemySprite(const PixmapArray &pa,
 			    Couple pos, Couple speed,
