@@ -1,8 +1,8 @@
-/*  $Id: EnemySprite.h,v 1.6 2004/05/02 04:55:47 sarrazip Exp $
+/*  $Id: EnemySprite.h,v 1.8 2007/12/22 23:03:35 sarrazip Exp $
     EnemySprite.h - Sprite that represents an enemy of the player.
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2007 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,21 +16,20 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #ifndef _H_EnemySprite
 #define _H_EnemySprite
 
+#include <flatzebra/Couple.h>
 #include <flatzebra/Sprite.h>
 
-using namespace flatzebra;
-
 class IngredientGroup;
 
 
-class EnemySprite : public Sprite
+class EnemySprite : public flatzebra::Sprite
 /*  Sprite that represents an enemy of the player.
 
     The "time to live" for this sprite is used to represent the time
@@ -42,8 +41,11 @@ class EnemySprite : public Sprite
 {
 public:
 
-    EnemySprite(const PixmapArray &pa, Couple pos, Couple speed,
-		Couple collBoxPos, Couple collBosSize) throw(int);
+    EnemySprite(const flatzebra::PixmapArray &pa,
+		flatzebra::Couple pos,
+		flatzebra::Couple speed,
+		flatzebra::Couple collBoxPos,
+		flatzebra::Couple collBosSize) throw(int);
     /*  Calls Sprite constructor.
     */
 
