@@ -1,8 +1,8 @@
-/*  $Id: IngredientGroup.h,v 1.6 2004/05/02 04:55:48 sarrazip Exp $
+/*  $Id: IngredientGroup.h,v 1.11 2007/12/22 23:03:35 sarrazip Exp $
     IngredientGroup.h - Group of sprites representing an "ingredient"
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2007 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #ifndef _H_IngredientGroup
@@ -28,8 +28,6 @@
 #include <assert.h>
 #include <vector>
 
-using namespace flatzebra;
-
 class IngredientSprite;
 class EnemySprite;
 
@@ -122,17 +120,19 @@ public:
 
     int  getVerticalTarget() const { return verticalTarget; }
 
-    void addCarriedEnemy(EnemySprite *s, int speedFactor);
+    void addCarriedEnemy(EnemySprite *s);
     /*  Adds 's' to this group's list of carried enemies.
 	Tells 's' that this group is its carrying group.
-	Give to 's' a downward speed of 'speedFactor'.
     */
     
-    SpriteList &getCarriedEnemies() { return carriedEnemies; }
+    const flatzebra::SpriteList &getCarriedEnemies() const { return carriedEnemies; }
+    flatzebra::SpriteList &getCarriedEnemies() { return carriedEnemies; }
     void clearCarriedEnemies() { carriedEnemies.clear(); }
 
+    int getBounceTime() const { return bounceTime; }
     int decBounceTime()
 	    { if (bounceTime != 0) bounceTime--; return bounceTime; }
+    int getStallTime() const { return stallTime; }
     int decStallTime()
 	    { if (stallTime != 0) stallTime--; return stallTime; }
 
@@ -140,19 +140,28 @@ public:
     size_t getNumFloorsToGo() const { return numFloorsToGo; }
     size_t decNumFloorsToGo()
 	    { if (numFloorsToGo != 0) numFloorsToGo--; return numFloorsToGo; }
-    Couple getCenterPos() const;
+    flatzebra::Couple getCenterPos() const;
 
-    bool isTopBun() const;
+    bool isTopBun() const { return topBunFlag; }
     /*  Indicates if this group represents a top bun.
 	When a top bun reaches the plate, the corresponding hamburger
 	is done.  When all hamburgers are done, the level is done.
     */
 
+    void restore(int bounceTime, int stallTime,
+			    State state, size_t numFloorsToGo)
+    {
+	this->bounceTime = bounceTime;
+	this->stallTime = stallTime;
+	this->state = state;
+	this->numFloorsToGo = numFloorsToGo;
+    }
+
 private:
 
     IngredientSprite *members[4];
     int verticalTarget;
-    SpriteList carriedEnemies;
+    flatzebra::SpriteList carriedEnemies;
     int bounceTime;
     int stallTime;
     State state;
