@@ -1,8 +1,8 @@
-/*  $Id: IngredientSprite.h,v 1.6 2004/05/02 04:55:48 sarrazip Exp $
+/*  $Id: IngredientSprite.h,v 1.9 2007/12/22 23:03:35 sarrazip Exp $
     IngredientSprite.h - Sprite that is part of an "ingredient"
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2004 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2007 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -16,8 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301, USA.
 */
 
 #ifndef _H_IngredientSprite
@@ -27,12 +27,10 @@
 
 #include <vector>
 
-using namespace flatzebra;
-
 class IngredientGroup;
 
 
-class IngredientSprite : public Sprite
+class IngredientSprite : public flatzebra::Sprite
 /*  This object does not own the 'IngredientGroup' to which it refers.
 */
 {
@@ -42,10 +40,17 @@ public:
 
     enum { LOWERING_DISTANCE = 4 };
 
-    IngredientSprite(const PixmapArray &pixmapArray,
-			const Couple &pos,
-			IngredientGroup *ig
-			/*, int ytarget*/ ) throw(int);
+    IngredientSprite(const flatzebra::PixmapArray &pixmapArray,
+			const flatzebra::Couple &pos,
+			IngredientGroup *ig) throw(int)
+      : flatzebra::Sprite(pixmapArray, pos,
+		flatzebra::Couple(0, 0), flatzebra::Couple(0, 0),
+		flatzebra::Couple(0, 5), flatzebra::Couple(24, 1)),
+	myGroup(ig),
+	lowered(false)
+    {
+	assert(ig != NULL);
+    }
 
     IngredientGroup *getGroup() const { return myGroup; }
 
